@@ -1,12 +1,18 @@
 from __future__ import annotations
 
+import abc
+import builtins
+import dataclasses
 import datetime
 import decimal
 import enum
 import inspect
 import ipaddress
+import numbers
 import pathlib
+import types
 import uuid
+from collections import namedtuple
 from operator import attrgetter
 from typing import (
     Any,
@@ -19,6 +25,10 @@ from typing import (
     Tuple,
     TypeVar,
     Iterable,
+    Iterator,
+    Hashable,
+    NamedTuple,
+    TYPE_CHECKING,
 )
 
 import typic
@@ -26,7 +36,19 @@ import typic.common
 import typic.util as util
 import typic.strict as strict
 
-from typic.compat import Final, ForwardRef, Literal, lru_cache
+from typic.compat import (
+    Final,
+    ForwardRef,
+    Literal,
+    lru_cache,
+    TypeGuard,
+    TypedDict,
+    Protocol,
+    Record,
+)
+
+if TYPE_CHECKING:
+    from typic.api import TypicObjectT
 
 ObjectT = TypeVar("ObjectT")
 """A type-alias for a python object.
@@ -47,6 +69,7 @@ str
 __all__ = (
     "BUILTIN_TYPES",
     "ObjectT",
+    "isabstract",
     "isbuiltininstance",
     "isbuiltintype",
     "isbuiltinsubtype",
@@ -54,8 +77,10 @@ __all__ = (
     "iscollectiontype",
     "isconstrained",
     "isdatetype",
+    "isdescriptor",
     "isenumtype",
     "isfinal",
+    "isforwardref",
     "isfromdictclass",
     "isfrozendataclass",
     "ishashable",
@@ -78,6 +103,7 @@ __all__ = (
     "istupletype",
     "istypeddict",
     "istypedtuple",
+    "isuniontype",
     "isuuidtype",
     "iswriteonly",
     "should_unwrap",
@@ -86,43 +112,34 @@ __all__ = (
 
 # Here we are with a manually-defined set of builtin-types.
 # This probably won't break anytime soon, but we shall see...
+BuiltInTypeT = Union[
+    int, bool, float, str, bytes, bytearray, list, set, frozenset, tuple, dict, None
+]
 BUILTIN_TYPES = frozenset(
-    (
-        int,
-        bool,
-        float,
-        str,
-        bytes,
-        bytearray,
-        list,
-        set,
-        frozenset,
-        tuple,
-        dict,
-        type(None),
-    )
+    (type(None), *(t for t in BuiltInTypeT.__args__ if t is not None))  # type: ignore
 )
 BUILTIN_TYPES_TUPLE = tuple(BUILTIN_TYPES)
+STDLibTypeT = Union[
+    BuiltInTypeT,
+    datetime.datetime,
+    datetime.date,
+    datetime.timedelta,
+    datetime.time,
+    decimal.Decimal,
+    ipaddress.IPv4Address,
+    ipaddress.IPv6Address,
+    pathlib.Path,
+    uuid.UUID,
+    uuid.SafeUUID,
+]
 STDLIB_TYPES = frozenset(
-    (
-        *BUILTIN_TYPES,
-        datetime.datetime,
-        datetime.date,
-        datetime.timedelta,
-        datetime.time,
-        decimal.Decimal,
-        ipaddress.IPv4Address,
-        ipaddress.IPv6Address,
-        pathlib.Path,
-        uuid.UUID,
-        uuid.SafeUUID,
-    )
+    (type(None), *(t for t in STDLibTypeT.__args__ if t is not None))  # type: ignore
 )
 STDLIB_TYPES_TUPLE = tuple(STDLIB_TYPES)
 
 
 @lru_cache(maxsize=None)
-def isbuiltintype(obj: Type[ObjectT]) -> bool:
+def isbuiltintype(obj: Type[ObjectT]) -> TypeGuard[Type[BuiltInTypeT]]:
     """Check whether the provided object is a builtin-type.
 
     Python stdlib and Python documentation have no "definitive list" of
@@ -163,7 +180,7 @@ def isbuiltintype(obj: Type[ObjectT]) -> bool:
 
 
 @lru_cache(maxsize=None)
-def isstdlibtype(obj: Type[ObjectT]) -> bool:
+def isstdlibtype(obj: Type[ObjectT]) -> TypeGuard[Type[STDLibTypeT]]:
     return (
         util.resolve_supertype(obj) in STDLIB_TYPES
         or util.resolve_supertype(type(obj)) in STDLIB_TYPES
@@ -171,7 +188,7 @@ def isstdlibtype(obj: Type[ObjectT]) -> bool:
 
 
 @lru_cache(maxsize=None)
-def isbuiltinsubtype(t: Type[ObjectT]) -> bool:
+def isbuiltinsubtype(t: Type[ObjectT]) -> TypeGuard[Type[BuiltInTypeT]]:
     """Check whether the provided type is a subclass of a builtin-type.
 
     Parameters
@@ -199,20 +216,20 @@ def isbuiltinsubtype(t: Type[ObjectT]) -> bool:
 
 
 @lru_cache(maxsize=None)
-def isstdlibsubtype(t: Type[ObjectT]) -> bool:
+def isstdlibsubtype(t: Type[ObjectT]) -> TypeGuard[Type[STDLibTypeT]]:
     return issubclass(util.resolve_supertype(t), STDLIB_TYPES_TUPLE)
 
 
-def isbuiltininstance(o: ObjectT) -> bool:
+def isbuiltininstance(o: ObjectT) -> TypeGuard[BuiltInTypeT]:
     return _isinstance(o, BUILTIN_TYPES_TUPLE)
 
 
-def isstdlibinstance(o: ObjectT) -> bool:
+def isstdlibinstance(o: ObjectT) -> TypeGuard[STDLibTypeT]:
     return _isinstance(o, STDLIB_TYPES_TUPLE)
 
 
 @lru_cache(maxsize=None)
-def isoptionaltype(obj: Type[ObjectT]) -> bool:
+def isoptionaltype(obj: Type[ObjectT]) -> TypeGuard[Optional]:
     """Test whether an annotation is :py:class`typing.Optional`, or can be treated as.
 
     :py:class:`typing.Optional` is an alias for `typing.Union[<T>, None]`, so both are
@@ -242,12 +259,17 @@ def isoptionaltype(obj: Type[ObjectT]) -> bool:
             type(None),
             None,
         }  # noqa: E721 - we don't know what args[-1] is, so this is safer
-        and getattr(obj, "__origin__", obj) in {Optional, Union, Literal}
+        and util.get_name(util.origin(obj)) in {"Optional", "Union", "Literal"}
     )
 
 
 @lru_cache(maxsize=None)
-def isreadonly(obj: Type[ObjectT]) -> bool:
+def isuniontype(obj: Type[ObjectT]) -> TypeGuard[Union]:
+    return util.get_name(util.origin(obj)) in {"Union", "UnionType"}
+
+
+@lru_cache(maxsize=None)
+def isreadonly(obj: Type[ObjectT]) -> TypeGuard[typic.common.ReadOnly]:
     """Test whether an annotation is marked as :py:class:`typic.ReadOnly`
 
     Parameters
@@ -286,14 +308,14 @@ def isfinal(obj: Type[ObjectT]) -> bool:
 
 
 @lru_cache(maxsize=None)
-def isliteral(obj: Type) -> bool:
+def isliteral(obj: Type) -> TypeGuard[Literal]:
     return util.origin(obj) is Literal or (
         obj.__class__ is ForwardRef and obj.__forward_arg__.startswith("Literal")
     )
 
 
 @lru_cache(maxsize=None)
-def iswriteonly(obj: Type[ObjectT]) -> bool:
+def iswriteonly(obj: Type[ObjectT]) -> TypeGuard[typic.common.WriteOnly]:
     """Test whether an annotation is marked as :py:class:`typic.WriteOnly`.
 
     Parameters
@@ -314,7 +336,7 @@ def iswriteonly(obj: Type[ObjectT]) -> bool:
 
 
 @lru_cache(maxsize=None)
-def isstrict(obj: Type[ObjectT]) -> bool:
+def isstrict(obj: Type[ObjectT]) -> TypeGuard[typic.Strict]:
     """Test whether an annotation is marked as :py:class:`typic.WriteOnly`.
 
     Parameters
@@ -326,16 +348,18 @@ def isstrict(obj: Type[ObjectT]) -> bool:
 
     >>> import typic
     >>> from typing import NewType
-    >>> typic.iswriteonly(typic.WriteOnly[str])
+    >>> typic.isstrict(typic.Strict[str])
     True
-    >>> typic.iswriteonly(NewType("Foo", typic.WriteOnly[str]))
+    >>> typic.isstrict(NewType("Foo", typic.Strict[str]))
     True
     """
     return util.origin(obj) is strict.Strict
 
 
 @lru_cache(maxsize=None)
-def isdatetype(obj: Type[ObjectT]) -> bool:
+def isdatetype(
+    obj: Type[ObjectT],
+) -> TypeGuard[Type[Union[datetime.datetime, datetime.date]]]:
     """Test whether this annotation is a a date/datetime object.
 
     Parameters
@@ -355,11 +379,11 @@ def isdatetype(obj: Type[ObjectT]) -> bool:
     >>> typic.isdatetype(NewType("Foo", datetime.datetime))
     True
     """
-    return _issubclass(util.origin(obj), (datetime.datetime, datetime.date))
+    return builtins.issubclass(util.origin(obj), (datetime.datetime, datetime.date))
 
 
 @lru_cache(maxsize=None)
-def istimetype(obj: Type[ObjectT]) -> bool:
+def istimetype(obj: Type[ObjectT]) -> TypeGuard[Type[datetime.time]]:
     """Test whether this annotation is a a date/datetime object.
 
     Parameters
@@ -377,11 +401,11 @@ def istimetype(obj: Type[ObjectT]) -> bool:
     >>> typic.istimetype(NewType("Foo", datetime.time))
     True
     """
-    return _issubclass(util.origin(obj), datetime.time)
+    return builtins.issubclass(util.origin(obj), datetime.time)
 
 
 @lru_cache(maxsize=None)
-def istimedeltatype(obj: Type[ObjectT]) -> bool:
+def istimedeltatype(obj: Type[ObjectT]) -> TypeGuard[Type[datetime.timedelta]]:
     """Test whether this annotation is a a date/datetime object.
 
     Parameters
@@ -399,11 +423,11 @@ def istimedeltatype(obj: Type[ObjectT]) -> bool:
     >>> typic.istimedeltatype(NewType("Foo", datetime.timedelta))
     True
     """
-    return _issubclass(util.origin(obj), datetime.timedelta)
+    return builtins.issubclass(util.origin(obj), datetime.timedelta)
 
 
 @lru_cache(maxsize=None)
-def isuuidtype(obj: Type[ObjectT]) -> bool:
+def isuuidtype(obj: Type[ObjectT]) -> TypeGuard[Type[uuid.UUID]]:
     """Test whether this annotation is a a date/datetime object.
 
     Parameters
@@ -425,26 +449,32 @@ def isuuidtype(obj: Type[ObjectT]) -> bool:
     >>> typic.isuuidtype(NewType("Foo", uuid.UUID))
     True
     """
-    return _issubclass(util.origin(obj), uuid.UUID)
+    return builtins.issubclass(util.origin(obj), uuid.UUID)
 
 
 _COLLECTIONS = {list, set, tuple, frozenset, dict, str, bytes}
 
 
 @lru_cache(maxsize=None)
-def isiterabletype(obj: Type[ObjectT]) -> bool:
+def isiterabletype(obj: Type[ObjectT]) -> TypeGuard[Type[Iterable]]:
     obj = util.origin(obj)
-    return _issubclass(obj, Iterable)
+    return builtins.issubclass(obj, Iterable)
 
 
 @lru_cache(maxsize=None)
-def istupletype(obj: Type[ObjectT]) -> bool:
+def isiteratortype(obj: Type[ObjectT]) -> TypeGuard[Type[Iterator]]:
+    obj = util.origin(obj)
+    return builtins.issubclass(obj, Iterator)
+
+
+@lru_cache(maxsize=None)
+def istupletype(obj: Type[ObjectT]) -> TypeGuard[Type[tuple]]:
     obj = util.origin(obj)
     return obj is tuple or issubclass(obj, tuple)
 
 
 @lru_cache(maxsize=None)
-def iscollectiontype(obj: Type[ObjectT]) -> bool:
+def iscollectiontype(obj: Type[ObjectT]) -> TypeGuard[Type[Collection]]:
     """Test whether this annotation is a subclass of :py:class:`typing.Collection`.
 
     Includes builtins.
@@ -472,11 +502,11 @@ def iscollectiontype(obj: Type[ObjectT]) -> bool:
     False
     """
     obj = util.origin(obj)
-    return obj in _COLLECTIONS or _issubclass(obj, Collection)
+    return obj in _COLLECTIONS or builtins.issubclass(obj, Collection)
 
 
 @lru_cache(maxsize=None)
-def ismappingtype(obj: Type[ObjectT]):
+def ismappingtype(obj: Type[ObjectT]) -> TypeGuard[Type[Mapping]]:
     """Test whether this annotation is a subtype of :py:class:`typing.Mapping`.
 
     Parameters
@@ -508,11 +538,15 @@ def ismappingtype(obj: Type[ObjectT]):
     True
     """
     obj = util.origin(obj)
-    return _issubclass(obj, dict) or _issubclass(obj, Mapping)
+    return (
+        builtins.issubclass(obj, dict)
+        or builtins.issubclass(obj, Mapping)
+        or builtins.issubclass(obj, Record)
+    )
 
 
 @lru_cache(maxsize=None)
-def isenumtype(obj: Type[ObjectT]) -> bool:
+def isenumtype(obj: Type[ObjectT]) -> TypeGuard[Type[enum.Enum]]:
     """Test whether this annotation is a subclass of :py:class:`enum.Enum`
 
     Parameters
@@ -534,7 +568,7 @@ def isenumtype(obj: Type[ObjectT]) -> bool:
 
 
 @lru_cache(maxsize=None)
-def isclassvartype(obj: Type[ObjectT]) -> bool:
+def isclassvartype(obj: Type[ObjectT]) -> TypeGuard[ClassVar]:
     """Test whether an annotation is a ClassVar annotation.
 
     Examples
@@ -570,17 +604,26 @@ def should_unwrap(obj: Type[ObjectT]) -> bool:
 
 
 @lru_cache(maxsize=None)
-def isfromdictclass(obj: Type[ObjectT]) -> bool:
+def isfromdictclass(obj: Type[ObjectT]) -> TypeGuard[_FromDict]:
     """Test whether this annotation is a class with a `from_dict()` method."""
     return inspect.isclass(obj) and hasattr(obj, "from_dict")
 
 
+class _FromDict(Protocol):
+    def from_dict(self, *args, **kwargs) -> _FromDict:
+        ...
+
+
 @lru_cache(maxsize=None)
-def isfrozendataclass(obj: Type[ObjectT]) -> bool:
+def isfrozendataclass(obj: Type[ObjectT]) -> TypeGuard[_FrozenDataclass]:
     """Test whether this is a dataclass and whether it's frozen."""
     return getattr(getattr(obj, "__dataclass_params__", None), "frozen", False)
 
 
+class _FrozenDataclass(Protocol):
+    __dataclass_params__: dataclasses._DataclassParams  # type: ignore
+
+
 _isinstance = isinstance
 
 
@@ -591,7 +634,7 @@ def _type_check(t) -> bool:
     return inspect.isclass(t)
 
 
-def isinstance(o: Any, t: Union[Type[ObjectT], Tuple[Type[ObjectT], ...]]) -> bool:
+def isinstance(o: Any, t: Union[Type, Tuple[Type, ...]]) -> bool:
     """A safer instance check...
 
     Validates that `t` is not an instance.
@@ -611,15 +654,10 @@ def isinstance(o: Any, t: Union[Type[ObjectT], Tuple[Type[ObjectT], ...]]) -> bo
     >>> typic.isinstance("", "")
     False
     """
-    return _type_check(t) and _isinstance(o, t)
+    return _type_check(t) and builtins.isinstance(o, t)
 
 
-_issubclass = issubclass
-
-
-def issubclass(
-    o: Type[Any], t: Union[Type[ObjectT], Tuple[Type[ObjectT], ...]]
-) -> bool:
+def issubclass(o: Type[Any], t: Union[Type, Tuple[Type, ...]]) -> bool:
     """A safer subclass check...
 
     Validates that `t` and/or `o` are not instances.
@@ -648,11 +686,11 @@ def issubclass(
     >>> typic.issubclass(MyStr, str())
     False
     """
-    return _type_check(t) and _type_check(o) and _issubclass(o, t)
+    return _type_check(t) and _type_check(o) and builtins.issubclass(o, t)
 
 
 @lru_cache(maxsize=None)
-def isconstrained(obj: Type[ObjectT]) -> bool:
+def isconstrained(obj: Type[ObjectT]) -> TypeGuard[_Constrained]:
     """Test whether a type is restricted.
 
     Parameters
@@ -676,10 +714,14 @@ def isconstrained(obj: Type[ObjectT]) -> bool:
     return hasattr(util.resolve_supertype(obj), "__constraints__")
 
 
+class _Constrained(Protocol):
+    __constraints__: typic.ConstraintsProtocolT
+
+
 __hashgetter = attrgetter("__hash__")
 
 
-def ishashable(obj: ObjectT) -> bool:
+def ishashable(obj: ObjectT) -> TypeGuard[Hashable]:
     """Check whether an object is hashable.
 
     An order of magnitude faster than :py:class:`isinstance` with
@@ -703,7 +745,7 @@ def ishashable(obj: ObjectT) -> bool:
 
 
 @lru_cache(maxsize=None)
-def istypeddict(obj: Type[ObjectT]) -> bool:
+def istypeddict(obj: Type[ObjectT]) -> TypeGuard[Type[TypedDict]]:
     """Check whether an object is a :py:class:`typing.TypedDict`.
 
     Parameters
@@ -724,12 +766,12 @@ def istypeddict(obj: Type[ObjectT]) -> bool:
     return (
         inspect.isclass(obj)
         and dict in {*inspect.getmro(obj)}
-        and hasattr(obj, "__annotations__")
+        and hasattr(obj, "__total__")
     )
 
 
 @lru_cache(maxsize=None)
-def istypedtuple(obj: Type[ObjectT]) -> bool:
+def istypedtuple(obj: Type[ObjectT]) -> TypeGuard[Type[NamedTuple]]:
     """Check whether an object is a "typed" tuple (:py:class:`typing.NamedTuple`).
 
     Parameters
@@ -755,7 +797,7 @@ def istypedtuple(obj: Type[ObjectT]) -> bool:
 
 
 @lru_cache(maxsize=None)
-def isnamedtuple(obj: Type[ObjectT]) -> bool:
+def isnamedtuple(obj: Type[ObjectT]) -> TypeGuard[namedtuple]:
     """Check whether an object is a "named" tuple (:py:func:`collections.namedtuple`).
 
     Parameters
@@ -774,12 +816,36 @@ def isnamedtuple(obj: Type[ObjectT]) -> bool:
     return inspect.isclass(obj) and issubclass(obj, tuple) and hasattr(obj, "_fields")
 
 
-def isproperty(obj) -> bool:
+def isforwardref(obj: Any) -> TypeGuard[ForwardRef]:
+    return obj.__class__ is ForwardRef
+
+
+def isproperty(obj) -> TypeGuard[types.GetSetDescriptorType]:
     return obj.__class__.__name__ in {"property", "cached_property"}
 
 
+def isdescriptor(obj) -> TypeGuard[types.GetSetDescriptorType]:
+    return (
+        hasattr(obj, "__get__")
+        or hasattr(obj, "__set__")
+        or hasattr(obj, "__set_name__")
+    )
+
+
 _ATTR_CHECKS = (inspect.isclass, inspect.isroutine, isproperty)
 
 
 def issimpleattribute(v) -> bool:
     return not any(c(v) for c in _ATTR_CHECKS)
+
+
+def isabstract(o) -> TypeGuard[abc.ABC]:
+    return inspect.isabstract(o) or o in _ABCS
+
+
+# Custom list of ABCs which incorrectly evaluate to false
+_ABCS = frozenset({numbers.Number})
+
+
+def istypicklass(obj) -> TypeGuard[TypicObjectT]:
+    return hasattr(obj, "__typic_fields__")
