@@ -13,9 +13,9 @@ from typing import (
 )
 
 try:
-    from typing import Final, TypedDict, Literal, Protocol  # type: ignore
+    from typing import Final, TypedDict, Literal, Protocol, Generic, TypeGuard, get_origin, get_args  # type: ignore
 except ImportError:  # pragma: nocover
-    from typing_extensions import Final, TypedDict, Literal, Protocol  # type: ignore
+    from typing_extensions import Final, TypedDict, Literal, Protocol, Generic, TypeGuard, get_origin, get_args  # type: ignore
 try:
     from typing import ForwardRef  # type: ignore
 except ImportError:  # pragma: nocover
@@ -29,13 +29,19 @@ except ImportError:  # pragma: nocover
 
 
 try:
-    from sqlalchemy.orm import registry as sqla_registry
+    from sqlalchemy.orm import registry as sqla_registry  # type: ignore
 except ImportError:  # pragma: nocover
 
     class sqla_registry:  # type: ignore
         ...
 
 
+try:  # pragma: nocover
+    from asyncpg import Record
+except (ImportError, ModuleNotFoundError):
+    Record = dict
+
+
 if sys.version_info < (3, 7):  # pragma: nocover
     if TYPE_CHECKING:
 
@@ -57,7 +63,7 @@ if sys.version_info < (3, 7):  # pragma: nocover
         return type_._eval_type(globalns, localns)
 
 
-elif sys.version_info >= (3, 9):
+elif sys.version_info >= (3, 9):  # pragma: nocover
     from typing import ForwardRef
 
     def evaluate_forwardref(
@@ -67,7 +73,7 @@ elif sys.version_info >= (3, 9):
         recursive_guard: set = None,
     ) -> Any:
         recursive_guard = recursive_guard or set()
-        return type_._evaluate(globalns, localns, recursive_guard)
+        return type_._evaluate(globalns, localns, recursive_guard)  # type: ignore
 
 
 else:  # pragma: nocover
@@ -79,7 +85,7 @@ else:  # pragma: nocover
         localns: Any,
         recursive_guard: set = None,
     ) -> Any:
-        return type_._evaluate(globalns, localns)
+        return type_._evaluate(globalns, localns)  # type: ignore
 
 
 if TYPE_CHECKING:
@@ -93,3 +99,35 @@ if TYPE_CHECKING:
 
 else:
     from functools import lru_cache
+
+
+if sys.version_info >= (3, 10):  # pragma: nocover
+    DATACLASS_KW_ONLY = DATACLASS_MATCH_ARGS = DATACLASS_NATIVE_SLOTS = True
+    from dataclasses import KW_ONLY  # type: ignore
+    from types import UnionType
+
+else:
+    DATACLASS_KW_ONLY = DATACLASS_MATCH_ARGS = DATACLASS_NATIVE_SLOTS = False
+    from typing import Union as UnionType
+
+    class _KW_ONLY_TYPE:
+        pass
+
+    KW_ONLY = _KW_ONLY_TYPE()
+
+__all__ = (
+    "Final",
+    "TypedDict",
+    "Literal",
+    "Protocol",
+    "TypeGuard",
+    "ForwardRef",
+    "SQLAMetaData",
+    "sqla_registry",
+    "evaluate_forwardref",
+    "lru_cache",
+    "DATACLASS_KW_ONLY",
+    "DATACLASS_MATCH_ARGS",
+    "DATACLASS_NATIVE_SLOTS",
+    "KW_ONLY",
+)
