@@ -1,5 +1,6 @@
 from __future__ import annotations
 
+import dataclasses
 import datetime
 import functools
 import inspect
@@ -23,11 +24,12 @@ from typing import (
     TYPE_CHECKING,
     Optional,
     Set,
+    NoReturn,
 )
 
 from pendulum import parse as dateparse, DateTime, instance
 
-from typic import checks, gen, constraints as const
+from typic import checks, gen
 from typic.strict import STRICT_MODE
 from typic.util import (
     safe_eval,
@@ -36,8 +38,12 @@ from typic.util import (
     get_defname,
     get_tag_for_types,
     get_unique_name,
+    get_name,
+    slotted,
 )
-from typic.common import DEFAULT_ENCODING, VAR_POSITIONAL, VAR_KEYWORD, ObjectT
+from typic.checks import ismappingtype
+from typic.common import DEFAULT_ENCODING, ObjectT
+from typic.compat import TypeGuard, Literal
 from .common import (
     DeserializerT,
     DeserializerRegistryT,
@@ -102,11 +108,13 @@ class DesFactory:
             re.Match,  # type: ignore
             type(None),
             _empty,
+            Callable,
+            abc.Callable,
         )
     )
     VNAME = "val"
     VTYPE = "vtype"
-    __DES_CACHE: Dict[str, Tuple[DeserializerT, "const.ValidatorT"]] = {}
+    __DES_CACHE: Dict[str, DeserializerT] = {}
     __USER_DESS: DeserializerRegistryT = deque()
 
     def __init__(self, resolver: Resolver):
@@ -164,10 +172,15 @@ class DesFactory:
                 b.l(f"return {self.VNAME}")
 
     @staticmethod
-    def _get_name(annotation: Annotation, constr: Optional[const.ConstraintsT]) -> str:
-        return get_defname("deserializer", (annotation, constr))
+    def _get_name(annotation: Annotation) -> str:
+        return get_defname("deserializer", annotation)
 
-    def _build_date_des(self, func: gen.Block, anno_name: str, annotation: Annotation):
+    def _build_date_des(self, context: BuildContext):
+        func, annotation, anno_name = (
+            context.func,
+            context.annotation,
+            context.anno_name,
+        )
         origin = annotation.resolved_origin
         # From an int
         with func.b(f"if isinstance({self.VNAME}, (int, float)):") as b:
@@ -219,7 +232,8 @@ class DesFactory:
             line = f"{self.VNAME} = dateparse({self.VNAME}, exact=True)"
             b.l(line, dateparse=dateparse)
 
-    def _build_time_des(self, func: gen.Block, anno_name: str, annotation: Annotation):
+    def _build_time_des(self, context: BuildContext):
+        func, anno_name = context.func, context.anno_name
         # From an int
         with func.b(f"if isinstance({self.VNAME}, (int, float)):") as b:
             b.l(f"{self.VNAME} = {anno_name}(int({self.VNAME}))")
@@ -236,9 +250,8 @@ class DesFactory:
         with func.b(f"if isinstance({self.VNAME}, date):", date=datetime.date) as b:
             b.l(f"{self.VNAME} = {anno_name}(0)")
 
-    def _build_timedelta_des(
-        self, func: gen.Block, anno_name: str, annotation: Annotation
-    ):
+    def _build_timedelta_des(self, context: BuildContext):
+        func, anno_name = context.func, context.anno_name
         # From an int
         with func.b(f"if isinstance({self.VNAME}, (int, float)):") as b:
             b.l(f"{self.VNAME} = {anno_name}(int({self.VNAME}))")
@@ -247,7 +260,8 @@ class DesFactory:
             line = f"{self.VNAME} = dateparse({self.VNAME}, exact=True)"
             b.l(line, dateparse=dateparse)
 
-    def _build_uuid_des(self, func: gen.Block, anno_name: str, annotation: Annotation):
+    def _build_uuid_des(self, context: BuildContext):
+        func, anno_name = context.func, context.anno_name
         self._add_type_check(func, anno_name)
         with func.b(f"if issubclass({self.VTYPE}, UUID):", UUID=uuid.UUID) as b:
             b.l(f"{self.VNAME} = {anno_name}(int={self.VNAME}.int)")
@@ -316,12 +330,12 @@ class DesFactory:
 
         return factory
 
-    def _build_text_des(
-        self,
-        func: gen.Block,
-        anno_name: str,
-        annotation: Annotation,
-    ):
+    def _build_text_des(self, context: BuildContext):
+        func, annotation, anno_name = (
+            context.func,
+            context.annotation,
+            context.anno_name,
+        )
         origin = annotation.resolved_origin
         # Encode for bytes
         if issubclass(origin, bytes):
@@ -336,24 +350,25 @@ class DesFactory:
                 b.l(f"{self.VNAME} = {self.VNAME}.decode({DEFAULT_ENCODING!r})")
         func.l(f"{self.VNAME} = {anno_name}({self.VNAME})")
 
-    def _build_builtin_des(
-        self,
-        func: gen.Block,
-        anno_name: str,
-        annotation: Annotation,
-    ):
+    def _build_builtin_des(self, context: BuildContext):
+        func, annotation, anno_name = (
+            context.func,
+            context.annotation,
+            context.anno_name,
+        )
         origin = annotation.resolved_origin
         if issubclass(origin, (str, bytes)):
-            self._build_text_des(func, anno_name, annotation)
+            self._build_text_des(context)
         elif checks.ismappingtype(origin):
-            self._build_mapping_des(func, anno_name, annotation)
+            self._build_mapping_des(context)
         elif checks.iscollectiontype(origin):
-            self._build_collection_des(func, anno_name, annotation)
+            self._build_collection_des(context)
         # bool, int, float...
         else:
             func.l(f"{self.VNAME} = {anno_name}({self.VNAME})")
 
-    def _build_pattern_des(self, func: gen.Block, anno_name: str):
+    def _build_pattern_des(self, context: BuildContext):
+        func, anno_name = context.func, context.anno_name
         func.l(
             f"{self.VNAME} = {self.VNAME} "
             f"if issubclass({self.VTYPE}, {anno_name}) "
@@ -365,16 +380,14 @@ class DesFactory:
         self._add_type_check(func, anno_name)
         func.l(f"{self.VNAME} = {anno_name}.from_dict({self.VNAME})")
 
-    def _build_typeddict_des(
-        self,
-        func: gen.Block,
-        anno_name: str,
-        annotation: Annotation,
-        *,
-        total: bool = True,
-        namespace: Type = None,
-    ):
-
+    def _build_typeddict_des(self, context: BuildContext):
+        func, annotation, namespace, anno_name = (
+            context.func,
+            context.annotation,
+            context.namespace,
+            context.anno_name,
+        )
+        total = getattr(context.annotation.resolved_origin, "__total__", True)
         with func.b(f"if issubclass({self.VTYPE}, Mapping):", Mapping=abc.Mapping) as b:
             fields_deser = {
                 x: self.resolver._resolve_from_annotation(
@@ -399,16 +412,16 @@ class DesFactory:
                 translate=self.resolver.translate,
             )
 
-    def _build_typedtuple_des(
-        self,
-        func: gen.Block,
-        anno_name: str,
-        annotation: Annotation,
-        namespace: Type = None,
-    ):
+    def _build_typedtuple_des(self, context: BuildContext):
+        func, annotation, anno_name = (
+            context.func,
+            context.annotation,
+            context.anno_name,
+        )
         with func.b(f"if issubclass({self.VTYPE}, Mapping):", Mapping=abc.Mapping) as b:
             if annotation.serde.fields:
-                self._build_typeddict_des(b, anno_name, annotation, namespace=namespace)
+                ctx = dataclasses.replace(context, func=b)
+                self._build_typeddict_des(ctx)
             else:
                 b.l(
                     f"{self.VNAME} = {anno_name}(**{self.VNAME})",
@@ -431,13 +444,13 @@ class DesFactory:
                 translate=self.resolver.translate,
             )
 
-    def _build_mapping_des(
-        self,
-        func: gen.Block,
-        anno_name: str,
-        annotation: Annotation,
-        namespace: Type = None,
-    ):
+    def _build_mapping_des(self, context: BuildContext):
+        func, annotation, namespace, anno_name = (
+            context.func,
+            context.annotation,
+            context.namespace,
+            context.anno_name,
+        )
         key_des, item_des = None, None
         args = annotation.args
         if args:
@@ -455,7 +468,9 @@ class DesFactory:
         kd_name = f"{anno_name}_key_des"
         it_name = f"{anno_name}_item_des"
         iterate = f"iterate({self.VNAME})"
+        iterate_values = f"iterate({self.VNAME}, values=True)"
         line = f"{anno_name}({iterate})"
+        line_values = f"{anno_name}({iterate_values})"
         if args or annotation.serde.fields_in:
             x, y = "x", "y"
             # If there are args & field mapping, get the correct field name
@@ -470,28 +485,35 @@ class DesFactory:
                 x = f"{kd_name}(x)"
                 y = f"{it_name}(y)"
             line = f"{anno_name}({{{x}: {y} for x, y in {iterate}}})"
+            line_values = f"{anno_name}({{{x}: {y} for x, y in {iterate_values}}})"
         # If we don't have nested annotations, we can short-circuit on valid inputs
         else:
             self._add_type_check(func, anno_name)
         # Write the lines.
-        func.l(
-            f"{self.VNAME} = {line}",
-            level=None,
-            **{
+        with func.b(f"if ismappingtype({self.VTYPE}):") as b:
+            b.l(f"{self.VNAME} = {line}")
+        with func.b("else:") as ob:
+            with ob.b("try:") as b:
+                b.l(f"{self.VNAME} = {line_values}")
+            with ob.b("except (TypeError, ValueError):") as b:
+                b.l(f"{self.VNAME} = {line}")
+        func.namespace.update(
+            {
                 kd_name: key_des,
                 it_name: item_des,
                 "Mapping": abc.Mapping,
                 "iterate": self.resolver.iterate,
-            },
+                "ismappingtype": ismappingtype,
+            }
         )
 
-    def _build_tuple_des(
-        self,
-        func: gen.Block,
-        anno_name: str,
-        annotation: Annotation,
-        namespace: Type = None,
-    ):
+    def _build_tuple_des(self, context: BuildContext):
+        func, annotation, namespace, anno_name = (
+            context.func,
+            context.annotation,
+            context.namespace,
+            context.anno_name,
+        )
         if annotation.args and annotation.args[-1] is not ...:
             item_des = {
                 ix: self.resolver.resolve(
@@ -515,20 +537,15 @@ class DesFactory:
                 },
             )
         else:
-            self._build_collection_des(
-                func=func,
-                anno_name=anno_name,
-                annotation=annotation,
-                namespace=namespace,
-            )
+            self._build_collection_des(context)
 
-    def _build_collection_des(
-        self,
-        func: gen.Block,
-        anno_name: str,
-        annotation: Annotation,
-        namespace: Type = None,
-    ):
+    def _build_collection_des(self, context: BuildContext):
+        func, annotation, namespace, anno_name = (
+            context.func,
+            context.annotation,
+            context.namespace,
+            context.anno_name,
+        )
         item_des = None
         it_name = f"{anno_name}_item_des"
         iterate = f"iterate({self.VNAME}, values=True)"
@@ -554,17 +571,18 @@ class DesFactory:
             },
         )
 
-    def _build_path_des(self, func: gen.Block, anno_name: str):
+    def _build_path_des(self, context: BuildContext):
+        func, anno_name = context.func, context.anno_name
         self._add_type_check(func, anno_name)
         func.l(f"{self.VNAME} = {anno_name}({self.VNAME})")
 
-    def _build_generic_des(
-        self,
-        func: gen.Block,
-        anno_name: str,
-        annotation: Annotation,
-        namespace: Type = None,
-    ):
+    def _build_user_type_des(self, context: BuildContext):
+        func, annotation, namespace, anno_name = (
+            context.func,
+            context.annotation,
+            context.namespace,
+            context.anno_name,
+        )
         serde = annotation.serde
         resolved = annotation.resolved
         self._add_type_check(func, anno_name)
@@ -616,8 +634,9 @@ class DesFactory:
         # Secondary branch - we have some other input for a user-defined class
         func.l("# Unknown path, just try casting it directly.")
         with func.b(
-            f"elif isbuiltinsubtype({self.VTYPE}):",
+            f"elif isbuiltinsubtype({self.VTYPE}) and not isnamedtuple({self.VTYPE}):",
             isbuiltinsubtype=checks.isbuiltinsubtype,
+            isnamedtuple=checks.isnamedtuple,
         ) as b:
             b.l(f"{self.VNAME} = {anno_name}({self.VNAME})")
         # Final branch - user-defined class for another user-defined class
@@ -651,50 +670,98 @@ class DesFactory:
         )
         return self._build_des(t_anno, func_name, namespace)
 
-    def _build_union_des(self, func: gen.Block, annotation: Annotation, namespace):
+    def _build_union_des(self, context: BuildContext):
+        func, annotation, namespace = (
+            context.func,
+            context.annotation,
+            context.namespace,
+        )
         # Get all types which we may coerce to.
         args = (*(a for a in annotation.args if a not in {None, Ellipsis, type(None)}),)
+        if not args:
+            return
+        # Add a type-check, but exclude str|bytes, since those are too permissive.
+        types = {a for a in args if a not in {str, bytes}}
+        if types:
+            with func.b(f"if {self.VTYPE} in types:", types=types) as b:
+                b.l(f"return {self.VNAME}")
         # Get all custom types, which may have discriminators
         targets = (*(a for a in args if not checks.isstdlibtype(a)),)
         # We can only build a tagged union deserializer if all args are valid
-        if args and args == targets:
-            # Try to collect the field which will be the discriminator.
-            # First, get a mapping of Type -> Proto & Type -> Fields
-            tagged = get_tag_for_types(targets)
-            # Just bail out if we can't find a key.
-            if not tagged:
-                func.l("# No-op, couldn't locate a discriminator key.")
-                return
-            # If we got a key, re-map the protocols to the value for each type.
-            deserializers = {
-                value: self.resolver.resolve(t, namespace=namespace)
-                for value, t in tagged.types_by_values
-            }
-            # Finally, build the deserializer
-            func.namespace.update(
-                tag=tagged.tag,
-                desers=deserializers,
-                empty=_empty,
+        if args != targets:
+            return self._build_generic_union_des(context)
+
+        # Try to collect the field which will be the discriminator.
+        # First, get a mapping of Type -> Proto & Type -> Fields
+        tagged = get_tag_for_types(targets)
+        # Just bail out if we can't find a key.
+        if not tagged:
+            return self._build_generic_union_des(context)
+        # If we got a key, re-map the protocols to the value for each type.
+        deserializers = {
+            value: self.resolver.resolve(t, namespace=namespace).transmute
+            for value, t in tagged.types_by_values
+        }
+        # Finally, build the deserializer
+        func.namespace.update(
+            tag=tagged.tag,
+            desers=deserializers,
+            empty=_empty,
+        )
+        with func.b(f"if issubclass({self.VTYPE}, Mapping):", Mapping=abc.Mapping) as b:
+            b.l(f"tag_value = {self.VNAME}.get(tag, empty)")
+        with func.b("else:") as b:
+            b.l(f"tag_value = getattr({self.VNAME}, tag, empty)")
+        with func.b("if tag_value in desers:") as b:
+            b.l(f"{self.VNAME} = desers[tag_value]({self.VNAME})")
+        with func.b("else:") as b:
+            b.l(
+                "raise ValueError("
+                'f"Value is missing field {tag!r} with one of '
+                '{(*desers,)}: {val!r}"'
+                ")"
             )
-            with func.b(
-                f"if issubclass({self.VTYPE}, Mapping):", Mapping=abc.Mapping
-            ) as b:
-                b.l(f"tag_value = {self.VNAME}.get(tag, empty)")
-            with func.b("else:") as b:
-                b.l(f"tag_value = getattr({self.VNAME}, tag, empty)")
-            with func.b("if tag_value in desers:") as b:
-                b.l(f"{self.VNAME} = desers[tag_value].transmute({self.VNAME})")
-            with func.b("else:") as b:
-                b.l(
-                    "raise ValueError("
-                    'f"Value is missing field {tag!r} with one of '
-                    '{(*desers,)}: {val!r}"'
-                    ")"
-                )
+
+    def _build_generic_union_des(self, context: BuildContext):
+        annotation, namespace, func = (
+            context.annotation,
+            context.namespace,
+            context.func,
+        )
+        annos = {
+            get_name(a): self.resolver.resolve(a, namespace=namespace)
+            for a in annotation.args
+            if a not in {None, Ellipsis, type(None)}
+        }
+        if annos:
+            desers = {f"{n}_des": p.transmute for n, p in annos.items()}
+            types = {n: p.annotation.resolved_origin for n, p in annos.items()}
+            ctx: Mapping[str, Union[Type, DeserializerT]] = {**types, **desers}
+            for name in annos:
+                # Can't do subclass checks with these...
+                if name in {"Literal", "Final"}:
+                    continue
+                with func.b(f"if issubclass({name}, {self.VTYPE}):") as b:
+                    b.l(f"return {name}_des({self.VNAME})")
+            for name in desers:
+                with func.b("try:") as b:
+                    b.l(f"return {name}({self.VNAME})")
+                with func.b("except (TypeError, ValueError, KeyError):") as b:
+                    b.l("pass")
+            func.namespace.update(ctx)
+            func.l(
+                "raise ValueError("
+                f'f"Value could not be deserialized into one of {(*annos,)}: {{val!r}}"'
+                ")",
+            )
+            return False
 
     def _build_des(  # noqa: C901
-        self, annotation: Annotation, func_name: str, namespace: Type = None
-    ) -> Callable:
+        self,
+        annotation: Annotation[Type[ObjectT]],
+        func_name: str,
+        namespace: Type = None,
+    ) -> DeserializerT[ObjectT]:
         args = annotation.args
         # Get the "origin" of the annotation.
         # For natives and their typing.* equivs, this will be a builtin type.
@@ -712,141 +779,63 @@ class DesFactory:
             return self._build_literal_des(annotation, func_name, namespace)
         with gen.Block(ns) as main:
             with main.f(func_name, main.param(f"{self.VNAME}")) as func:
+                needs_return = None
+                context = BuildContext(annotation, ns, anno_name, func, namespace)
                 if origin not in self.UNRESOLVABLE:
+                    # Set our top-level sanity checks.
                     self._set_checks(func, anno_name, annotation)
-                    if origin is Union:
-                        self._build_union_des(func, annotation, namespace)
-                    elif checks.isdatetype(origin):
-                        self._build_date_des(func, anno_name, annotation)
-                    elif checks.istimetype(origin):
-                        self._build_time_des(func, anno_name, annotation)
-                    elif checks.istimedeltatype(origin):
-                        self._build_timedelta_des(func, anno_name, annotation)
-                    elif checks.isuuidtype(origin):
-                        self._build_uuid_des(func, anno_name, annotation)
-                    elif origin in {Pattern, re.Pattern}:  # type: ignore
-                        self._build_pattern_des(func, anno_name)
-                    elif issubclass(origin, pathlib.Path):
-                        self._build_path_des(func, anno_name)
-                    elif not args and checks.isbuiltintype(origin):
-                        self._build_builtin_des(func, anno_name, annotation)
-                    elif checks.isfromdictclass(origin):
-                        self._build_fromdict_des(func, anno_name)
-                    elif checks.isenumtype(origin):
-                        self._build_builtin_des(func, anno_name, annotation)
-                    elif checks.istypeddict(origin):
-                        self._build_typeddict_des(
-                            func,
-                            anno_name,
-                            annotation,
-                            total=origin.__total__,  # type: ignore
-                            namespace=namespace,
-                        )
-                    elif checks.istypedtuple(origin) or checks.isnamedtuple(origin):
-                        self._build_typedtuple_des(
-                            func, anno_name, annotation, namespace=namespace
-                        )
-                    elif not args and checks.isbuiltinsubtype(origin):
-                        self._build_builtin_des(func, anno_name, annotation)
-                    elif checks.ismappingtype(origin):
-                        self._build_mapping_des(
-                            func, anno_name, annotation, namespace=namespace
-                        )
-                    elif checks.istupletype(origin):
-                        self._build_tuple_des(
-                            func, anno_name, annotation, namespace=namespace
-                        )
-                    elif checks.iscollectiontype(origin):
-                        self._build_collection_des(
-                            func, anno_name, annotation, namespace=namespace
-                        )
-                    else:
-                        self._build_generic_des(
-                            func, anno_name, annotation, namespace=namespace
-                        )
-                func.l(f"{gen.Keyword.RET} {self.VNAME}")
+                    # Move through our queue.
+                    for check, handler in self._HANDLERS.items():
+                        # If this is a valid type for this handler,
+                        #   write the deserializer.
+                        if check(origin, args):
+                            needs_return = handler(self, context)
+                            break
+                # If the deserializer doesn't contain a return statement, add one.
+                if needs_return is not False:
+                    func.l(f"{gen.Keyword.RET} {self.VNAME}")
         deserializer = main.compile(ns=ns, name=func_name)
         return deserializer
 
-    def _check_varargs(
-        self, anno: Annotation, des: DeserializerT, validator: const.ValidatorT
-    ) -> Tuple[DeserializerT, "const.ValidatorT"]:
-        if anno.parameter.kind == VAR_POSITIONAL:
-            __des = des
-            __validator = validator
-
-            def des(__val, *, __des=__des):
-                return (*(__des(x) for x in __val),)
-
-            def validator(value, *, field: str = None, __validator=__validator):  # type: ignore
-                return (*(__validator(x, field=field) for x in value),)
-
-        elif anno.parameter.kind == VAR_KEYWORD:
-            __des = des
-            __validator = validator
-
-            def des(__val, *, __des=__des):
-                return {x: __des(y) for x, y in __val.items()}
-
-            def validator(value, *, field: str = None, __validator=__validator):  # type: ignore
-                return {x: __validator(y, field=field) for x, y in value.items()}
-
-        return des, validator
-
-    def _finalize_validator(
-        self,
-        constr: Optional[const.ConstraintsT],
-    ) -> const.ValidatorT:
-        def validate(value, *, field: str = None):
-            return value
-
-        if constr:
-            validate = constr.validate  # noqa: F811
-
-        return validate  # type: ignore
-
-    def _finalize_deserializer(
-        self,
-        anno: Annotation,
-        des: DeserializerT,
-        constr: Optional[const.ConstraintsT],
-    ) -> Tuple[DeserializerT, const.ValidatorT]:
-        # Determine how to run in "strict-mode"
-        validator = self._finalize_validator(constr)
-        # Handle *args and **kwargs
-        des, validator = self._check_varargs(anno, des, validator)
-        # If we have type constraints, override the deserializer for strict annotations.
-        if isinstance(constr, (const.TypeConstraints, const.LiteralConstraints)):
-            if anno.strict:
-                des = validator  # type: ignore
-            elif isinstance(constr, const.LiteralConstraints):
-                __d = des
-
-                def des(val: Any, *, __d=__d, __v=validator) -> ObjectT:
-                    return __v(__d(val))
-
-        # Otherwise
-        else:
-            # In strict mode, we validate & coerce if there are constraints
-            if anno.strict and constr and constr.coerce:
-                __d = des
-
-                def des(val: Any, *, __d=__d, __v=validator) -> ObjectT:
-                    return __d(__v(val))
-
-            elif anno.strict:
-                des = validator  # type: ignore
-
-        return des, validator
+    # Order is IMPORTANT! This is a FIFO queue.
+    _HANDLERS: Mapping[HandlerCheckT, BuildHandlerT] = {
+        # Special handler for Unions...
+        lambda origin, args: checks.isuniontype(origin): _build_union_des,
+        # Non-intersecting types (order doesn't matter here.
+        lambda origin, args: checks.isdatetype(origin): _build_date_des,
+        lambda origin, args: checks.istimetype(origin): _build_time_des,
+        lambda origin, args: checks.istimedeltatype(origin): _build_timedelta_des,
+        lambda origin, args: checks.isuuidtype(origin): _build_uuid_des,
+        lambda origin, args: origin in {Pattern, re.Pattern}: _build_pattern_des,
+        lambda origin, args: issubclass(origin, pathlib.Path): _build_path_des,
+        # MUST come before subtype check.
+        lambda origin, args: (
+            not args and checks.isbuiltintype(origin)
+        ): _build_builtin_des,
+        # Psuedo-structured containers, should check before generics.
+        lambda origin, args: checks.istypeddict(origin): _build_typeddict_des,
+        lambda origin, args: checks.istypedtuple(origin): _build_typedtuple_des,
+        lambda origin, args: checks.isnamedtuple(origin): _build_typedtuple_des,
+        lambda origin, args: (
+            not args and checks.isbuiltinsubtype(origin)
+        ): _build_builtin_des,
+        # A mapping is a collection so must come before that check.
+        lambda origin, args: checks.ismappingtype(origin): _build_mapping_des,
+        # A tuple is a collection so must come before that check.
+        lambda origin, args: checks.istupletype(origin): _build_tuple_des,
+        # Generic collection handler
+        lambda origin, args: checks.iscollectiontype(origin): _build_collection_des,
+        # Catch-all for custom user types (user-defined classes).
+        lambda origin, args: True: _build_user_type_des,
+    }
 
     def factory(
         self,
-        annotation: Annotation,
-        constr: Optional[const.ConstraintsT] = None,
+        annotation: Annotation[Type[ObjectT]],
         namespace: Type = None,
-    ) -> Tuple[DeserializerT, "const.ValidatorT"]:
+    ) -> DeserializerT[ObjectT]:
         annotation.serde = annotation.serde or SerdeConfig()
-        key = self._get_name(annotation, constr)
+        key = self._get_name(annotation)
         if key in self.__DES_CACHE:
             return self.__DES_CACHE[key]
         deserializer: Optional[DeserializerT] = None
@@ -856,10 +845,19 @@ class DesFactory:
                 break
         if not deserializer:
             deserializer = self._build_des(annotation, key, namespace)
+        self.__DES_CACHE[key] = deserializer
+        return deserializer
 
-        deserializer, validator = self._finalize_deserializer(
-            annotation, deserializer, constr
-        )
-        self.__DES_CACHE[key] = (deserializer, validator)
 
-        return deserializer, validator
+@slotted(dict=False, weakref=True)
+@dataclasses.dataclass
+class BuildContext:
+    annotation: Annotation
+    ns: Mapping[str, Any]
+    anno_name: str
+    func: gen.Block
+    namespace: Optional[Type] = None
+
+
+HandlerCheckT = Callable[[Type[ObjectT], Tuple[Any, ...]], TypeGuard[Type[ObjectT]]]
+BuildHandlerT = Callable[[DesFactory, BuildContext], Union[NoReturn, Literal[False]]]
