@@ -18,6 +18,7 @@ from typing import (
     TypeVar,
     Iterator,
     Dict,
+    Iterable,
 )
 
 from typic import checks, constraints as constr, util, strict as st
@@ -30,7 +31,7 @@ from typic.common import (
     Case,
     ReadOnly,
 )
-from typic.compat import ForwardRef, lru_cache
+from typic.compat import lru_cache
 from typic.ext import json
 from typic.strict import StrictModeT
 from .binder import Binder
@@ -44,10 +45,12 @@ from .common import (
     DelayedSerdeProtocol,
     ForwardDelayedAnnotation,
     DelayedAnnotation,
-    AnnotationT,
     DeserializerT,
     EncoderT,
     DecoderT,
+    TranslatorT,
+    PrimitiveT,
+    FieldIteratorT,
 )
 from .des import DesFactory
 from .ser import SerFactory
@@ -103,7 +106,7 @@ class Resolver:
             The value to be transmuted
         """
         resolved: SerdeProtocol = self.resolve(annotation)
-        transmuted: ObjectT = resolved.transmute(value)  # type: ignore
+        transmuted: ObjectT = resolved.transmute(value)
 
         return transmuted
 
@@ -125,8 +128,9 @@ class Resolver:
         target
             The higher-order class to translate into.
         """
-        resolved: SerdeProtocol = self.resolve(type(value))
-        return resolved.translate(value, target)  # type: ignore
+        t = value.__class__
+        resolved: SerdeProtocol = self.resolve(t)
+        return resolved.translate(value, target)
 
     def validate(
         self, annotation: Type[ObjectT], value: Any, *, transmute: bool = False
@@ -145,11 +149,11 @@ class Resolver:
         resolved: SerdeProtocol = self.resolve(annotation)
         value = resolved.validate(value)
         if transmute:
-            return resolved.transmute(value)  # type: ignore
+            return resolved.transmute(value)
         return value
 
     def iterate(
-        self, obj, *, values: bool = False
+        self, obj, *, values: bool = False, exclude: Iterable[str] = ()
     ) -> Iterator[Union[Tuple[str, Any], Any]]:
         """Iterate over the fields of an object.
 
@@ -159,6 +163,8 @@ class Resolver:
             The object to iterate over
         values
             Whether to only yield values of an object's fields. (defaults False)
+        exclude
+            Proactively ignore any fields on the object
         """
         t = obj.__class__
         # Extract the type of the enum value if this is an Enum.
@@ -166,7 +172,7 @@ class Resolver:
         if checks.isenumtype(t):
             obj = obj.value
             t = obj.__class__
-        iterator = self.translator.iterator(t, values=values)
+        iterator = self.translator.iterator(t, values=values, exclude=(*exclude,))
         return iterator(obj)
 
     def coerce_value(
@@ -186,7 +192,14 @@ class Resolver:
     def delayed(self, t: Type) -> bool:
         return getattr(t, "__delayed__", False)
 
-    def primitive(self, obj: Any, lazy: bool = False, name: util.ReprT = None) -> Any:
+    def primitive(
+        self,
+        obj: ObjectT,
+        *,
+        lazy: bool = False,
+        name: util.ReprT = None,
+        flags: SerdeFlags = None,
+    ) -> PrimitiveT:
         """A method for converting an object to its primitive equivalent.
 
         Useful for encoding data to JSON.
@@ -222,13 +235,13 @@ class Resolver:
         """
         t = obj.__class__
         if checks.isenumtype(t):
-            obj = obj.value
+            obj = obj.value  # type: ignore
             t = obj.__class__
-        proto: SerdeProtocol = self.resolve(t)
+        proto: SerdeProtocol = self.resolve(t, flags=flags)
         return proto.primitive(obj, lazy=lazy, name=name)  # type: ignore
 
     def tojson(
-        self, obj: Any, *, indent: int = 0, ensure_ascii: bool = False, **kwargs
+        self, obj: ObjectT, *, indent: int = 0, ensure_ascii: bool = False, **kwargs
     ) -> str:
         """A method for dumping any object to a valid JSON string.
 
@@ -248,45 +261,32 @@ class Resolver:
         >>> import dataclasses
         >>> import enum
         >>> typic.tojson("foo")
-        '"foo"'
+        b'"foo"'
         >>> typic.tojson(("foo",))
-        '["foo"]'
+        b'["foo"]'
         >>> typic.tojson(datetime.datetime(1970, 1, 1))
-        '"1970-01-01T00:00:00"'
+        b'"1970-01-01T00:00:00"'
         >>> typic.tojson(b"foo")
-        '"foo"'
+        b'"foo"'
         >>> typic.tojson(ipaddress.IPv4Address("0.0.0.0"))
-        '"0.0.0.0"'
-        >>> typic.tojson(re.compile("[0-9]"))
-        '"[0-9]"'
+        b'"0.0.0.0"'
         >>> typic.tojson(uuid.UUID(int=0))
-        '"00000000-0000-0000-0000-000000000000"'
-        >>> @dataclasses.dataclass
-        ... class Foo:
-        ...     bar: str = 'bar'
-        ...
-        >>> typic.tojson(Foo())
-        '{"bar":"bar"}'
-        >>> class Enum(enum.Enum):
-        ...     FOO = "foo"
-        ...
-        >>> typic.tojson(Enum.FOO)
-        '"foo"'
+        b'"00000000-0000-0000-0000-000000000000"'
         """
         t = obj.__class__
         if checks.isenumtype(t):
-            obj = obj.value
+            obj = obj.value  # type: ignore
             t = obj.__class__
         proto: SerdeProtocol = self.resolve(t)
         return proto.tojson(obj, indent=indent, ensure_ascii=ensure_ascii, **kwargs)
 
     def decode(
-        self, annotation: Type[ObjectT], value: Any, decoder: DecoderT, **kwargs
+        self, annotation: Type[ObjectT], value: Any, decoder: DecoderT[bytes], **kwargs
     ) -> ObjectT:
         proto: SerdeProtocol = self.resolve(annotation)
         return proto.transmute(decoder(value, **kwargs))  # type: ignore
 
-    def encode(self, obj: Any, encoder: EncoderT, **kwargs) -> bytes:
+    def encode(self, obj: Any, encoder: EncoderT[PrimitiveT], **kwargs) -> bytes:
         t = obj.__class__
         if checks.isenumtype(t):
             obj = obj.value
@@ -295,15 +295,13 @@ class Resolver:
         return encoder(proto.primitive(obj), **kwargs)  # type: ignore
 
     @lru_cache(maxsize=None)
-    def _get_configuration(self, origin: Type, flags: "SerdeFlags") -> "SerdeConfig":
+    def _get_configuration(self, origin: Type, flags: SerdeFlags) -> SerdeConfig:
         if hasattr(origin, SERDE_FLAGS_ATTR):
             flags = getattr(origin, SERDE_FLAGS_ATTR)
         # Get all the annotated fields
         params = util.safe_get_params(origin)
         # This is probably a builtin and has no signature
-        fields: Dict[
-            str, Union[Annotation, DelayedAnnotation, ForwardDelayedAnnotation]
-        ] = {}
+        fields: Dict[str, Annotation] = {}
         hints = util.cached_type_hints(origin)
         for name, t in hints.items():
             fields[name] = self.annotation(
@@ -381,7 +379,7 @@ class Resolver:
         flags: "SerdeFlags" = None,
         default: Any = EMPTY,
         namespace: Type = None,
-    ) -> AnnotationT:
+    ) -> Annotation[Type[ObjectT]]:
         """Get a :py:class:`Annotation` for this type.
 
         Unlike a :py:class:`ResolvedAnnotation`, this does not provide access to a
@@ -443,13 +441,13 @@ class Resolver:
                     "for more information."
                 )
         # The type definition doesn't exist yet.
-        if use.__class__ is ForwardRef:
+        if checks.isforwardref(use):
             module = self.__module__
             # Ideally we have a namespace from a parent class/function to the field
             if namespace:
                 module = namespace.__module__
 
-            return ForwardDelayedAnnotation(
+            fda = ForwardDelayedAnnotation(
                 ref=use,
                 resolver=self,
                 _name=name,
@@ -461,13 +459,14 @@ class Resolver:
                 module=module,
                 frame=inspect.currentframe(),
             )
+            return cast(Annotation, fda)
         # The type definition is recursive or within a recursive loop.
         elif use is namespace or use in self.__stack:
             # If detected via stack, we can remove it now.
             # Otherwise we'll cause another recursive loop.
             if use in self.__stack:
                 self.__stack.remove(use)
-            return DelayedAnnotation(
+            da = DelayedAnnotation(
                 type=use,
                 resolver=self,
                 _name=name,
@@ -477,6 +476,7 @@ class Resolver:
                 flags=flags,
                 default=default,
             )
+            return cast(Annotation, da)
         # Otherwise, add this type to the stack to prevent a recursive loop from elsewhere.
         if not checks.isstdlibtype(use):
             self.__stack.add(use)
@@ -496,15 +496,54 @@ class Resolver:
             static=is_static,
             serde=serde,
         )
-        anno.translator = functools.partial(self.translator.factory, anno)  # type: ignore
+        anno.translator = cast(
+            TranslatorT, functools.partial(self.translator.factory, anno)
+        )
         return anno
 
+    @staticmethod
+    def _finalize_deserializer(
+        annotation: Annotation[Type[ObjectT]],
+        deserializer: DeserializerT[ObjectT],
+        constraints: constr.ConstraintsProtocolT[ObjectT],
+    ) -> Tuple[DeserializerT[ObjectT], constr.ValidateT[ObjectT]]:
+        # Set the default returns.
+        validator = constraints.validate
+        rdeserializer = deserializer
+        # If we're in "strict" mode, we want to default to only validation.
+        if annotation.strict or checks.isabstract(annotation.resolved_origin):
+            rdeserializer = cast(DeserializerT[ObjectT], validator)
+
+        # If we have type constraints, we'll bail out early.
+        # If we have literal constraints, we must validate as a part of coercion.
+        if isinstance(constraints, (constr.TypeConstraints, constr.LiteralConstraints)):
+            if isinstance(constraints, constr.LiteralConstraints):
+                d = deserializer
+
+                def des(val: Any, *, __d=d, __v=validator) -> ObjectT:
+                    return __v(__d(val))
+
+                rdeserializer = cast(DeserializerT[ObjectT], des)
+
+            return rdeserializer, validator
+        # Finally, if we're in "strict" mode, but the constraint needs validation prior
+        #   to coercion, we should inject the validator within the deserializer.
+        if annotation.strict and constraints.coerce:
+            d = deserializer
+
+            def des(val: Any, *, __d=d, __v=validator) -> ObjectT:
+                return __d(__v(val))
+
+            rdeserializer = cast(DeserializerT[ObjectT], des)
+
+        return rdeserializer, validator
+
     def _resolve_from_annotation(
         self,
-        anno: AnnotationT,
+        anno: Annotation[Type[ObjectT]],
         *,
         namespace: Type = None,
-    ) -> SerdeProtocol:
+    ) -> SerdeProtocol[ObjectT]:
         if anno in self.__cache:
             return self.__cache[anno]
         if isinstance(anno, (DelayedAnnotation, ForwardDelayedAnnotation)):
@@ -514,66 +553,45 @@ class Resolver:
         constraints = constr.get_constraints(
             anno.resolved, nullable=anno.optional, cls=namespace
         )
-        deserializer, validator = self.des.factory(
-            anno, constraints, namespace=namespace
+        deserializer = self.des.factory(anno, namespace=namespace)
+        deserializer, validator = self._finalize_deserializer(
+            annotation=anno, deserializer=deserializer, constraints=constraints
         )
         # Build the serializer
-        serializer: Optional[SerializerT] = self.ser.factory(anno)
+        serializer = self.ser.factory(anno)
         # Put it all together
         proto = self._build_protocol(
             annotation=anno,
             constraints=constraints,
             deserializer=deserializer,
-            serializer=serializer,
             validator=validator,
+            serializer=serializer,
         )
         self.__cache[anno] = proto
         return proto
 
     def _build_protocol(
         self,
-        annotation: Annotation,
-        constraints: constr.ConstraintsT,
         *,
-        deserializer: DeserializerT = None,
-        validator: constr.ValidatorT = None,
-        serializer: SerializerT = None,
-    ) -> SerdeProtocol:
-        # Pass through if for some reason there's no coercer.
-        deserialize = deserializer or (lambda o: o)
-        # Set the validator
-        validate = validator or (lambda value, *, field=None, **kwargs: value)
-        # Set the serializer
-        serialize = serializer or (lambda o, lazy=False, name=None: o)
-
-        def tojson(
-            val: ObjectT,
-            *,
-            indent: int = 0,
-            ensure_ascii: bool = False,
-            __prim=serialize,
-            __dumps=json.dumps,
-            **kwargs,
-        ) -> str:
-            return __dumps(
-                __prim(val, lazy=True),
-                indent=indent,
-                ensure_ascii=ensure_ascii,
-                **kwargs,
-            )
-
+        annotation: Annotation[Type[ObjectT]],
+        constraints: constr.ConstraintsProtocolT[ObjectT],
+        deserializer: DeserializerT[ObjectT],
+        validator: constr.ValidateT[ObjectT],
+        serializer: SerializerT[ObjectT],
+    ) -> SerdeProtocol[ObjectT]:
+        tojson = json.get_tojson(serializer)
         tojson.__qualname__ = f"{SerdeProtocol.__name__}.{tojson.__name__}"
         tojson.__module__ = SerdeProtocol.__module__
 
         # Set the encoder/decoder protocols.
         # Default to JSON for wire-format
-        encode = tojson
+        encode: EncoderT = cast(EncoderT, tojson)
         if annotation.serde.encoder:
 
             def encode(  # type: ignore
                 val: ObjectT,
                 *,
-                __prim=serialize,
+                __prim=serializer,
                 __encode=annotation.serde.encoder,
                 **kwargs,
             ) -> bytes:
@@ -583,13 +601,13 @@ class Resolver:
             encode.__module__ = self.__class__.__module__
 
         # Default to JSON for wire-format
-        decode = deserialize
+        decode: DecoderT = cast(DecoderT, deserializer)
         if annotation.serde.decoder:
 
-            def decode(
+            def decode(  # type: ignore
                 val: bytes,
                 *,
-                __trans=deserialize,
+                __trans=deserializer,
                 __decode=annotation.serde.decoder,
                 **kwargs,
             ) -> ObjectT:
@@ -600,26 +618,59 @@ class Resolver:
 
         # Create the translator
         def translate(
-            val: Any, target: Type[_T], *, __factory=annotation.translator
+            value: ObjectT, target: Type[_T], *, __factory=annotation.translator
         ) -> _T:
             trans = __factory(target)
-            return trans(val)
+            return trans(value)
 
         translate.__qualname__ = f"{SerdeProtocol.__name__}.{translate.__name__}"
         translate.__module__ = SerdeProtocol.__module__
 
+        # Create the iterator, if possible.
+        try:
+            iterator = self._iterator_from_annotation(annotation)
+            iterator.__qualname__ = f"{SerdeProtocol.__name__}.{iterator.__name__}"
+            iterator.__module__ = SerdeProtocol.__module__
+        # Default to lazy iteration, if not.
+        except TypeError:
+            iterator = cast(FieldIteratorT, self.iterate)
+
         return SerdeProtocol(
             annotation=annotation,
             constraints=constraints,
-            deserialize=deserialize,
+            deserialize=deserializer,
             decode=decode,
-            serialize=serialize,
-            encode=encode,  # type: ignore
-            validate=validate,
-            translate=translate,  # type: ignore
-            tojson=tojson,  # type: ignore
+            serialize=serializer,
+            encode=encode,
+            validate=validator,
+            translate=cast(TranslatorT, translate),
+            tojson=cast(EncoderT, tojson),
+            iterate=iterator,
         )
 
+    def _iterator_from_annotation(
+        self, annotation: Annotation[Type[ObjectT]]
+    ) -> FieldIteratorT[ObjectT]:
+        exclude = (*(annotation.serde.flags.exclude or ()),)
+        fiterator = self.translator.iterator(
+            annotation.resolved_origin, relaxed=True, exclude=exclude
+        )
+        viterator = self.translator.iterator(
+            annotation.resolved_origin,
+            values=True,
+            relaxed=True,
+            exclude=exclude,
+        )
+
+        def iterator(
+            o: ObjectT, *, values: bool = False, __fields=fiterator, __values=viterator
+        ) -> Iterator[Union[Tuple[str, Any], Any]]:
+            if values:
+                return __values(o)
+            return __fields(o)
+
+        return cast(FieldIteratorT, iterator)
+
     @lru_cache(maxsize=None)
     def resolve(
         self,
@@ -631,7 +682,7 @@ class Resolver:
         is_optional: bool = None,
         is_strict: bool = None,
         namespace: Type = None,
-    ) -> SerdeProtocol:
+    ) -> SerdeProtocol[ObjectT]:
         """Get a :py:class:`SerdeProtocol` from a given annotation or type.
 
         Parameters
@@ -706,9 +757,7 @@ class Resolver:
         :py:class:`SerdeProtocol`
         """
 
-        if not any(
-            (inspect.ismethod(obj), inspect.isfunction(obj), inspect.isclass(obj))
-        ):
+        if not callable(obj):
             obj = obj.__class__
 
         hints = util.cached_type_hints(obj)
